#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  buckwalter.py
#  A  class to handle hepburn translateration
#
#  Copyright 2018 Abdelkrime Aries <kariminfo0@gmail.com>
#
#  ---- AUTHORS ----
#  2018	Abdelkrime Aries <kariminfo0@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

hiragana = [
	u"が",
	u"ぎ",
	u"ぐ",
	u"げ",
	u"ご",
	u"ざ",
	u"じ",
	u"じ",
	u"ず",
	u"ぜ",
	u"ぞ",
	u"だ",
	u"ぢ",
	u"ぢ",
	u"づ",
	u"で",
	u"ど",
	u"ば",
	u"び",
	u"ぶ",
	u"べ",
	u"ぼ",
	u"ぱ",
	u"ぴ",
	u"ぷ",
	u"ぺ",
	u"ぽ",
	u"ふぁ",
	u"ふぉ",
	u"ふぃ",
	u"か",
	u"き",
	u"く",
	u"け",
	u"こ",
	u"た",
	u"ち",
	u"つ",
	u"て",
	u"と",
	u"さ",
	u"し",
	u"し",
	u"す",
	u"せ",
	u"そ",
	u"な",
	u"に",
	u"ぬ",
	u"ね",
	u"の",
	u"は",
	u"ひ",
	u"ふ",
	u"へ",
	u"ほ",
	u"ま",
	u"み",
	u"む",
	u"め",
	u"も",
	u"や",
	u"ゆ",
	u"よ",
	u"ら",
	u"り",
	u"る",
	u"れ",
	u"ろ",
	u"わ",
	u"を",
	u"あ",
	u"い",
	u"う",
	u"え",
	u"お",
	u"ん",
	u"ゃ",
	u"ぇ",
	u"ゅ",
	u"ぃ",
	u"ょ"
]
trans_tab = [ # https://en.wikipedia.org/wiki/Hepburn_romanization
	"ga",
	"gi",
	"gu",
	"ge",
	"go",
	"za",
	"ji",
	"zi",
	"zu",
	"ze",
	"zo",
	"da",
	"ji",
	"di",
	"zu",
	"de",
	"do",
	"ba",
	"bi",
	"bu",
	"be",
	"bo",
	"pa",
	"pi",
	"pu",
	"pe",
	"po",
	"fa",
	"fo",
	"fi",
	"ka",
	"ki",
	"ku",
	"ke",
	"ko",
	"ta",
	"chi",
	"tsu",
	"te",
	"to",
	"sa",
	"shi",
	"si",
	"su",
	"se",
	"so",
	"na",
	"ni",
	"nu",
	"ne",
	"no",
	"ha",
	"hi",
	"fu",
	"he",
	"ho",
	"ma",
	"mi",
	"mu",
	"me",
	"mo",
	"ya",
	"yu",
	"yo",
	"ra",
	"ri",
	"ru",
	"re",
	"ro",
	"wa",
	"wo",
	"a",
	"i",
	"u",
	"e",
	"o",
	"n",
	"xya",
	"xye",
	"xyu",
	"xyi",
	"xyo"
]

loneUnTrans = {
	"ku": "く",
	"su": "す",
	"nu": "ぬ",
	"fu": "ふ",
	"hu": "ふ",
	"mu": "む",
	"yu": "ゆ",
	"ru": "る",
	"gu": "ぐ",
	"zu": "ず",
	"du": "づ",
	"bu": "ぶ",
	"pu": "ぷ"
}


from lang_trans import L2LTrans
import re


def post_trans(word):
	res = re.sub(ur"ix", "", word)
	return re.sub(ur"(っ+)(.)", lambda m: (m.group(2) * (1 + len(m.group(1)))), res)

def tsu_replace_call(m):
	c = m.group(1)
	try:
		"aeuio".index(c) # if no exception
		return m.group()
	except ValueError:
		rep_char = u"っ"
		if c == "n":
			rep_char = u"ん"
		nbr = len(m.group())/len(c) - 1
		return (rep_char * nbr) + c

def tsu_replace(word):
	return re.sub(r"(sh|ch|.)\1+", tsu_replace_call, word)

def xya2jap_call(m):
	res = m.group()
	try:
		i = trans_tab.index(m.group(1) + "i") # can generate ValueError
		res = hiragana[i]
		i = trans_tab.index("xy" + m.group(2)) # can generate ValueError
		res += hiragana[i]
		return res
	except ValueError:
		return res

def xya2jap(word):
	return re.sub(r"(sh|ch|.)y([aeuio])", xya2jap_call, word)

def post_untrans_call(m):
	key = m.group()
	more_than_one = (len(m.group()) > 1)
	if more_than_one:
		if key == u"tう":
			return "つ"
		key = key[:-1]
	key += "u"
	res = ""
	if key in loneUnTrans:
		res = loneUnTrans[key]
	if more_than_one:
		res += u"う"
	return res

def post_untrans(word):
	return re.sub(ur"[a-z][う]?", post_untrans_call, word)

class Hepburn(L2LTrans):

	def __init__(self):
		L2LTrans.__init__(self, hiragana, trans_tab)

	@staticmethod
	def pre_trans(word):
		return re.sub(ur"([しちじぢ])([ゃぇゅょ])", lambda m: (
		(trans_tab[hiragana.index(m.group(1))] + trans_tab[hiragana.index(m.group(2))]).replace("ixy", "")
		), word)

	def transliterate(self, word):
		res = Hepburn.pre_trans(word)
		res = L2LTrans.transliterate(self, res)
		return post_trans(res)

	@staticmethod
	def pre_untrans(word):
		res = tsu_replace(word)
		res = re.sub(r"(sh|ch|j)([aeuo])", xya2jap_call, res)
		return xya2jap(res)

	def untransliterate(self, word):
		res = Hepburn.pre_untrans(word)
		res = L2LTrans.untransliterate(self, res)
		return post_untrans(res)


hepburn = Hepburn()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  transliteration.py
#  An abstract class  for language transliteration
#
#  Copyright 2017-2018 Abdelkrime Aries <kariminfo0@gmail.com>
#
#  ---- AUTHORS ----
#  2017-2018	Abdelkrime Aries <kariminfo0@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


import sys
import abc

if sys.version_info >= (3, 4):
	ABC = abc.ABC
else:
	ABC = abc.ABCMeta('ABC', (), {})


class Trans(ABC):

	def __init__(self):
		pass

	def trans(self, word):
		return self.transliterate(word)

	@abc.abstractmethod
	def transliterate(self, word):
		return word

	def untrans(self, word):
		return self.untransliterate(word)

	@abc.abstractmethod
	def untransliterate(self, word):
		return word

class MapTrans(Trans):

	def __init__(self, the_map):
		self.the_map = the_map

	def transliterate(self, word):
		res = ""
		if len(word) < len(self.the_map):
			for char in word:
				res += self.the_map.get(char, char)
		else:
			res = word
			for k,v in self.the_map.items():
				res = res.replace(k, v)

		return res

	def untransliterate(self, word):
		res = word
		for k,v in self.the_map.items():
			res = res.replace(v, k)

		return res


class L2LTrans(Trans):

	def __init__(self, src, dst):
		self.src = src
		self.dst = dst

	def transliterate(self, word):
		res = word
		for i, v in enumerate(self.src):
			res = res.replace(v, self.dst[i])
		return res

	def untransliterate(self, word):
		res = word
		for i, v in enumerate(self.dst):
			res = res.replace(v, self.src[i])
		return res



import os
from setuptools import setup

def readme():
    try:
        with open('README.rst') as f:
            return f.read()
    except IOError:
        return ''

setup(
    name = "lang-trans",
    version = "0.5.0",
    author = "Abdelkrime Aries",
    author_email = "kariminfo0@gmail.com",
    description = ("Python transliteration library"),
    license = "Apache-2.0",
    keywords = "transliteration nlp languages romanization",
    url = "https://github.com/kariminf/lang-trans",
    packages=['lang_trans'],
    long_description=readme()
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  buckwalter.py
#  A  class to handle buckwalter translateration
#
#  Copyright 2017-2018 Abdelkrime Aries <kariminfo0@gmail.com>
#
#  ---- AUTHORS ----
#  2017-2018	Abdelkrime Aries <kariminfo0@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

buckwalter_map =	{
		u"ا": "A", # alif
		u"ب": "b", # ba
		u"ت": "t", # ta
		u"ث": "v", # tha
		u"ج": "j", # jim
		u"ح": "H", # Ḥa
		u"خ": "x", # kha
		u"د": "d", # dal
		u"ذ": "*", # dhal
		u"ر": "r", # ra
		u"ز": "z", # zin
		u"س": "s", # sin
		u"ش": "$", # shin
		u"ص": "S", # ṣad
		u"ض": "D", # Ḍad
		u"ط": "T", # Ṭa
		u"ظ": "Z", # Ẓa
		u"ع": "E", # 'Ayn
		u"غ": "g", # ghayn
		u"ف": "f", # fa
		u"ق": "q", # qaf
		u"ك": "k", # kaf
		u"ل": "l", # lam
		u"م": "m", # mim
		u"ن": "n", # nun
		u"ه": "h", # ha
		u"و": "w", # waw
		u"ي": "y", # ya
		#hamza
		u"ء": "'", # lone hamza
		u"أ": ">", # hamza on alif
		u"إ": "<", # hamza below alif
		u"ؤ": "&", # hamza on waw
		u"ئ": "}", # hamza on ya
		#alif
		u"آ": "|", # madda on alif
		u"ٱ": "{", # alif alwasla
		u"\u0670": "`", # dagger alif
		u"ى": "Y", # alif maqsura
		#harakat
		u"\u064E": "a", # fatha
		u"\u064F": "u", # Damma
		u"\u0650": "i", # kasra
		u"\u064B": "F", # fathatayn
		u"\u064C": "N", # dammatayn
		u"\u064D": "K", # kasratayn
		u"\u0651": "~", # shadda
		u"\u0640": "_", # tatwil
		#others
		u"ة": "p", # ta marbuta
		u"\u0652": "o", # sukun
}


iso233_map =	{
		u"ا": u"ʾ", # alif
		u"ب": u"b", # ba
		u"ت": u"t", # ta
		u"ث": u"ṯ", # tha
		u"ج": u"ǧ", # jim
		u"ح": u"ḥ", # Ḥa
		u"خ": u"ẖ", # kha
		u"د": u"d", # dal
		u"ذ": u"ḏ", # dhal
		u"ر": u"r", # ra
		u"ز": u"z", # zin
		u"س": u"s", # sin
		u"ش": u"š", # shin
		u"ص": u"ṣ", # ṣad
		u"ض": u"ḍ", # Ḍad
		u"ط": u"ṭ", # Ṭa
		u"ظ": u"ẓ", # Ẓa
		u"ع": u"ʿ", # 'Ayn
		u"غ": u"ġ", # ghayn
		u"ف": u"f", # fa
		u"ق": u"q", # qaf
		u"ك": u"k", # kaf
		u"ل": u"l", # lam
		u"م": u"m", # mim
		u"ن": u"n", # nun
		u"ه": u"h", # ha
		u"و": u"w", # waw
		u"ي": u"y", # ya
		#hamza
		u"ء": u"ˌ", # lone hamza
		u"أ": u"ˈ", # hamza on alif
		u"إ": u"ʾ", # hamza below alif
		u"ؤ": u"ˈ", # hamza on waw
		u"ئ": u"ˈ", # hamza on ya
		#alif
		u"\u0622": u"ʾ", # madda on alif
		u"\u0671": u"ʾ", # alif alwasla
		u"\u0670": u"ʾ", # dagger alif
		u"ى": u"ỳ", # alif maqsura
		u"ة": u"ẗ", # ta marbuta
}

arabic = [
	u"ث", # tha
	u"ح", # Ḥa
	u"خ", # kha
	u"ذ", # dhal
	u"ش", # shin
	u"ص", # ṣad
	u"ض", # Ḍad
	u"ط", # Ṭa
	u"ظ", # Ẓa
	u"غ", # ghayn
	u"ب", # ba
	u"ت", # ta
	u"ج", # jim
	u"د", # dal
	u"ر", # ra
	u"ز", # zin
	u"س", # sin
	u"ع", # 'Ayn
	u"ف", # fa
	u"ق", # qaf
	u"ك", # kaf
	u"ل", # lam
	u"م", # mim
	u"ن", # nun
	u"ه", # ha
	u"و", # waw
	u"ي", # ya
	u"أ", # hamza on alif
	u"إ", # hamza below alif
	u"ؤ", # hamza on waw
	u"ئ", # hamza on ya
	u"آ", # madda on alif
	u"\u0671", # alif alwasla
	u"\u0670", # dagger alif
	u"ى", # alif maqsura
	u"\u064B", # fathatayn
	u"\u064C", # dammatayn
	u"\u064D", # kasratayn
	u"\u064E", # fatha
	u"\u064F", # Damma
	u"\u0650", # kasra
	u"\u0651", # shadda
	u"\u0640", # tatwil
	u"ة", # ta marbuta
	u"\u0652", # sukun
	u"ء", # lone hamza
	u"ا" # alif
]

arabtex_tab = [
	"_t", # tha
	".h", # Ḥa
	"_h", # kha
	"_d", # dhal
	"^s", # shin
	".s", # ṣad
	".d", # Ḍad
	".t", # Ṭa
	".z", # Ẓa
	".g", # ghayn
	"b", # ba
	"t", # ta
	"j", # jim
	"d", # dal
	"r", # ra
	"z", # zin
	"s", # sin
	"`", # 'Ayn
	"f", # fa
	"q", # qaf
	"k", # kaf
	"l", # lam
	"m", # mim
	"n", # nun
	"h", # ha
	"w", # waw or "U"
	"y", # ya or "I"

	"'", # hamza on alif
	"'i", # hamza below alif
	"'", # hamza on waw
	"'", # hamza on ya

	"'A", # madda on alif
	"|", # alif alwasla <<<<<<
	"_a", # dagger alif <<<<<<
	"_A", # alif maqsura

	"aN", # fathatayn
	"uN", # dammatayn
	"iN", # kasratayn
	"a", # fatha
	"u", # Damma
	"i", # kasra
	"xx", # shadda
	"--", # tatwil

	"T", # ta marbuta
	"\"", # sukun

	"'", # lone hamza
	"A" # alif
  ]

from lang_trans import MapTrans, L2LTrans

class Buckwalter(MapTrans):

	def __init__(self):
		MapTrans.__init__(self, buckwalter_map)

buckwalter = Buckwalter()

class ArabTex(L2LTrans):

	def __init__(self):
		L2LTrans.__init__(self, arabic, arabtex_tab)

arabtex = ArabTex()

class Iso233(MapTrans):

	def __init__(self):
		MapTrans.__init__(self, iso233_map)

iso233 = Iso233()
